/***********************************************************************************************
 * Chickencoop door controller based on github project https://github.com/chrisuthe/CoopDoor   *
 *                                                                                             *
 * updated with support from facebook groups Arduino and Arduino Belgium                       *
 *                                                                                             *
 * Door open/close on calculated sunset/sunrise with Dusk2Dawn library                         *
 * Switch on pin 3 to override and keep door closed                                            *
 *                                                                                             *
 * Geert Wanten - V1 - 25/07/2020                                                              * 
 ***********************************************************************************************/


#include <math.h>
#include <Dusk2Dawn.h>
#include <EEPROM.h>
#include <Wire.h>
#include "RTClib.h"



//Define what pins are used
const int doorUpSensorPin = 4;        // reedswitch door open
const int doorDownSensorPin = 5;      // reedswitch door down
const int doorDownMotorPin = 10;      // wind motor down
const int DoorUpMotorPin = 9;         // wind motor up
const int builtInLED = 12;            // LED idicating door closed (reedswitch on pin 5)
const byte CloseDoorManual = 3;       // Close door by hand

int buttonstateclose = 0;
int currentMins;
int sunrise;
int sunset;
DateTime now;

//Are We Debugging?
const bool debug = true;
//Do we need to set/adjust RTC?
const bool setRTC = false;

//Define RTC
RTC_DS1307 rtc;

//define the location / timezone for dusk2dawn (this is yourcity, countrycode 2 digit)
// change yourcity an fill in coordinates and timezone
Dusk2Dawn yourcity(0, 0, 0); 

// the setup function runs once when you press reset or power the board
void setup() {
  
  //Set the mode the pins will operate in.
    pinMode(DoorUpMotorPin, OUTPUT);
    pinMode(doorDownMotorPin, OUTPUT);
    pinMode(CloseDoorManual, INPUT); 

  
  //Set Serial for Debugging
  if (debug) {
    Serial.begin(9600);

    if (!rtc.begin()) {
    Serial.println("Couldn't find RTC");
    while (1);
    }
    
  rtc.begin();
  //   //Lets just set the date/time when setRTC = true.
  if (setRTC)
  {
   //write PC date and time while uploading and when setRTC = true
   rtc.adjust(DateTime(F(__DATE__), F(__TIME__)));

   //uncomment line 71 to write date and time manually into RTC when line 38 setRTC = true. For testing sunset/sunrise
    //rtc.adjust(DateTime(2020, 7, 23, 7, 9, 20));       
  }
 }
  
  
  
  //switches setup
  pinMode(doorUpSensorPin, INPUT);
  pinMode(doorDownSensorPin, INPUT);
  pinMode(builtInLED, OUTPUT);
    
  Serial.println("FinishedSetup");
  if (digitalRead(doorUpSensorPin) == HIGH) 
  {
   Serial.println("Door is open");
   }

   if (digitalRead(doorDownSensorPin) == HIGH)
   {
    Serial.println("Door is closed");
   }
   delay(3000);
}


// the loop function runs over and over again until power down or reset
void loop() {
  delay(2000);
  if (debug){
    Serial.println();
    Serial.println("LoopHead");
  }
  //Get the Date/Time from the RTC
  now = rtc.now();
  //Get Sunrise/Sunset for the current year/month/day as INT's that equate to minutes from midnight that he sunrise/sunset will occur (THE TRUE is passing in Daylight Savings Time!)
  sunrise = Kessel.sunrise(now.year(), now.month(), now.day(), true);
  sunset = Kessel.sunset(now.year(), now.month(), now.day(), true);
  if (debug) {
    Serial.println();
    Serial.print(now.year());
    Serial.print('/');
    Serial.print(now.month());
    Serial.print('/');
    Serial.print(now.day());
    Serial.print(" - ");
    Serial.print(now.hour());
    Serial.print(':');
    Serial.print(now.minute());
    Serial.print(':');
    Serial.print(now.second());
  }

    
    buttonstateclose = digitalRead(CloseDoorManual);
    if(buttonstateclose == HIGH){
      ManualClose();
    }


  
  //Lets get add the "now" Minutes and "now" hours*60 to see how many minutes from midnight we are
  currentMins = ((now.hour()) * 60) + (now.minute());
  Serial.println(currentMins);

  //lets start comparisons, if the door should be up....
  //delay of 30 minutes after sunset time to make sure all chickens are inside before closing the door. 
  if ((sunrise + 30 < currentMins && currentMins < sunset + 30) && ( buttonstateclose != HIGH))
  {
    Serial.println("Door should be up");
    while ( digitalRead(doorUpSensorPin) == LOW )
    raiseDoor();
    }

  else
  {
    Serial.println("Door should be down");
    while ( digitalRead(doorDownSensorPin) == LOW )
    lowerDoor();
   }

  


  int DoorUp = digitalRead(doorUpSensorPin);
  int DoorDown = digitalRead(doorDownSensorPin);
  /* This is just temporary switch debug code*/
  if (DoorUp == HIGH)
  {
    Serial.println("Door Opened All The Way");
    digitalWrite(builtInLED, LOW);
    stopDoor();
    
  }

  else
  {
    digitalRead(doorDownSensorPin == HIGH);
    Serial.println("Door Closed All The Way");
    digitalWrite(builtInLED, HIGH);
    stopDoor();
    
  } 
}



//Wind the Door Up
void raiseDoor() {
  digitalWrite(DoorUpMotorPin, HIGH);
  digitalWrite(doorDownMotorPin, LOW);
  if (debug) {
    Serial.println("Door Raising");
  }
}


//Wind The Door Down
void lowerDoor() {
  digitalWrite(doorDownMotorPin, HIGH);
  digitalWrite(DoorUpMotorPin, LOW);
  if (debug) {
    Serial.println("Door Lowering");
  }
}



//Stop the Door
void stopDoor() {
  digitalWrite(DoorUpMotorPin, LOW);
  digitalWrite(doorDownMotorPin, LOW);
  if (debug) {
    Serial.println("Door Stop");
  }
}


//Close door manual
void ManualClose() { 
  digitalWrite(doorDownMotorPin, HIGH);
  digitalWrite(DoorUpMotorPin, LOW);
  if (debug) {
    Serial.println("Door Closing Manual");
  while ( digitalRead(doorDownSensorPin) == LOW)
    digitalWrite(builtInLED, HIGH);
    stopDoor(); 
  }
 }
